import numpy as np
from enum import Enum
 
class ScoringMethod(str, Enum):
    mse = "mse"

class SimilarityComparer:
    def mse(img_a, img_b):
        err = np.sum((img_a.astype("float") - img_b.astype("float")) ** 2)
        err /= float(img_a.shape[0] * img_a.shape[1])
        return err
    
# TODO: Implement another method to compare images


from pydantic import BaseModel
from scorer import ScoringMethod, SimilarityComparer
from model import Frame
from utils import Utils

class Config(BaseModel):
    interval: int = 10
    threshold: float = 1000.0
    score: ScoringMethod = "mse"
    
class Engine:
    def __init__(self, args: Config = Config()):
        self.args = args
        if self.args.score == "mse":
            self.score = SimilarityComparer.mse
        
    # def compare_frames_1(self, frame_a: Frame) -> float:
    #     """ 
    #     method 1: compare each frame with distinct frames that have been found ; slower
    #     """
    #     return [self.score(frame_a.data, frame_b.data) for frame_b in self.distinct_frames]
    
    def __call__(self, path: str) -> list[Frame]:
        """ 
        method 2: compare each frame with the previous frame ; faster
        """
        self.video = Utils.read_video(path, self.args.interval)
        self.distinct_frames = [self.video.frames[0]]
        for i in range(1, len(self.video.frames)):
            mse_value = self.score(self.video.frames[i].data, self.video.frames[i-1].data)
            if mse_value > self.args.threshold:
                self.distinct_frames.append(self.video.frames[i])
        
        return self.distinct_frames

class Utils(Utils):
    pass


import numpy as np
from pydantic import BaseModel, validator

class Frame(BaseModel):
    data: np.ndarray
    pos_id: int
    
    class Config:
        arbitrary_types_allowed = True
        
    @validator('data')
    def check_data(cls, v):
        assert isinstance(v, np.ndarray), 'data must be a numpy array'
        return v

class Video(BaseModel):
    frames: list[Frame]
    fps: float
    total_frames: int
    duration: float
    
    @validator('frames')
    def check_frames(cls, v):
        assert len(v) > 0, 'frames must have at least one frame'
        return v
    
    @validator('fps')
    def check_fps(cls, v):
        assert v > 0, 'fps must be greater than 0'
        return v
    
    @validator('total_frames')
    def check_total_frames(cls, v):
        assert v > 0, 'total_frames must be greater than 0'
        return v
    
    @validator('duration')
    def check_duration(cls, v):
        assert v > 0, 'duration must be greater than 0'
        return v


from rec2slide import Engine, Config, Utils

def main():
    config = Config(
        interval=20,
        threshold=1000.0,
        score="mse"
    )
    
    engine = Engine(config)
    result = engine("./data/example.mp4")
    Utils.frames2pdf(result, "./data/example.pdf")
    
if __name__ == "__main__":
    main()


from cv2 import VideoCapture, CAP_PROP_FPS
from model import Video, Frame
from PIL import Image

class Utils:
    @staticmethod
    def read_video(path: str, interval: int) -> Video:
        cap = VideoCapture(path)
        fps = cap.get(CAP_PROP_FPS)
        frames = []
        ret, frame = cap.read()
        while ret:
            frame_id = int(cap.get(1))
            if frame_id % interval == 0:
                frames.append(Frame(data=frame, pos_id=frame_id))
            ret, frame = cap.read()
        return Video(
                    frames=frames, 
                    fps=fps, 
                    total_frames=len(frames), 
                    duration=len(frames)/fps
                )
        
    @staticmethod
    def frames2pdf(result: Video, output_path: str) -> None:
        pil_imgs = [Image.fromarray(frame.data) for frame in result]
        pil_imgs[0].save(
            output_path,
            "PDF",
            resolution=100.0,
            save_all=True,
            append_images=pil_imgs[1:],
        )

    @staticmethod
    def frames2jpg(result: Video, output_path: str) -> None:
        for i, frame in enumerate(result):
            Image.fromarray(frame.data).save(f"{output_path}/frame_{i}.jpg")
    
    @staticmethod
    def frames2png(result: Video, output_path: str) -> None:
        for i, frame in enumerate(result):
            Image.fromarray(frame.data).save(f"{output_path}/frame_{i}.png")
